@@ -84,7 +84,11 @@ func (id PublicID) String() string {
 }
 
 func (id1 PublicID) Less(id2 PublicID) bool {
-	return slices.Compare(id1[:], id2[:]) < 0
+	return id1.Compare(id2) < 0
+}
+
+func (id1 PublicID) Compare(id2 PublicID) int {
+	return slices.Compare(id1[:], id2[:])
 }
 
 func (id PublicID) IsZero() bool {
